from datetime import datetime

from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC
from discord import EntityType

from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.scheduled_event_repository_abc import ScheduledEventRepositoryABC
from bot_data.model.scheduled_event import ScheduledEvent
from bot_data.model.scheduled_event_interval_enum import ScheduledEventIntervalEnum
from bot_data.model.user_role_enum import UserRoleEnum
from bot_graphql.abc.query_abc import QueryABC
from bot_core.service.permission_service import PermissionService


class ScheduledEventMutation(QueryABC):
    def __init__(
        self,
        servers: ServerRepositoryABC,
        scheduled_events: ScheduledEventRepositoryABC,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        permissions: PermissionService,
    ):
        QueryABC.__init__(self, "ScheduledEventMutation")

        self._servers = servers
        self._scheduled_events = scheduled_events
        self._bot = bot
        self._db = db
        self._permissions = permissions

        self.set_field("createScheduledEvent", self.resolve_create_scheduled_event)
        self.set_field("updateScheduledEvent", self.resolve_update_scheduled_event)
        self.set_field("deleteScheduledEvent", self.resolve_delete_scheduled_event)

    def resolve_create_scheduled_event(self, *_, input: dict):
        server = self._servers.get_server_by_id(input["serverId"])
        self._can_user_mutate_data(server, UserRoleEnum.moderator)

        scheduled_event = ScheduledEvent(
            ScheduledEventIntervalEnum(input["interval"]),
            input["name"],
            input["description"] if "description" in input else None,
            input["channelId"] if "channelId" in input else None,
            datetime.strptime(input["startTime"], "%Y-%m-%dT%H:%M:%S.%fZ"),
            datetime.strptime(input["endTime"], "%Y-%m-%dT%H:%M:%S.%fZ") if "endTime" in input else None,
            EntityType(int(input["entityType"])),
            input["location"] if "location" in input else None,
            server,
        )

        self._scheduled_events.add_scheduled_event(scheduled_event)
        self._db.save_changes()

        def get_new_scheduled_event(srn: ScheduledEvent):
            return (
                srn.interval == scheduled_event.interval
                and srn.name == scheduled_event.name
                and srn.description == scheduled_event.description
            )

        return (
            self._scheduled_events.get_scheduled_events_by_server_id(scheduled_event.server.id)
            .where(get_new_scheduled_event)
            .last()
        )

    def resolve_update_scheduled_event(self, *_, input: dict):
        scheduled_event = self._scheduled_events.get_scheduled_event_by_id(input["id"])
        self._can_user_mutate_data(scheduled_event.server, UserRoleEnum.moderator)

        scheduled_event.interval = (
            ScheduledEventIntervalEnum(input["interval"]) if "interval" in input else scheduled_event.interval
        )
        scheduled_event.name = input["name"] if "name" in input else scheduled_event.name
        scheduled_event.description = input["description"] if "description" in input else scheduled_event.description
        scheduled_event.channel_id = input["channelId"] if "channelId" in input else scheduled_event.channel_id
        scheduled_event.start_time = (
            datetime.strptime(input["startTime"], "%Y-%m-%dT%H:%M:%S.%fZ")
            if "startTime" in input
            else scheduled_event.start_time
        )
        scheduled_event.end_time = (
            datetime.strptime(input["endTime"], "%Y-%m-%dT%H:%M:%S.%fZ")
            if "endTime" in input
            else scheduled_event.end_time
        )
        scheduled_event.entity_type = (
            EntityType(int(input["entityType"])) if "entityType" in input else scheduled_event.entity_type
        )
        scheduled_event.location = input["location"] if "location" in input else scheduled_event.location

        self._scheduled_events.update_scheduled_event(scheduled_event)
        self._db.save_changes()

        scheduled_event = self._scheduled_events.get_scheduled_event_by_id(input["id"])
        return scheduled_event

    def resolve_delete_scheduled_event(self, *_, id: int):
        scheduled_event = self._scheduled_events.get_scheduled_event_by_id(id)
        self._can_user_mutate_data(scheduled_event.server, UserRoleEnum.admin)

        self._scheduled_events.delete_scheduled_event(scheduled_event)
        self._db.save_changes()

        return scheduled_event
