from cpl_core.configuration import ConfigurationABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.achievement_repository_abc import AchievementRepositoryABC
from bot_data.abc.auto_role_repository_abc import AutoRoleRepositoryABC
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.game_server_repository_abc import GameServerRepositoryABC
from bot_data.abc.known_user_repository_abc import KnownUserRepositoryABC
from bot_data.abc.level_repository_abc import LevelRepositoryABC
from bot_data.abc.scheduled_event_repository_abc import ScheduledEventRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.short_role_name_repository_abc import ShortRoleNameRepositoryABC
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.abc.user_joined_game_server_repository_abc import (
    UserJoinedGameServerRepositoryABC,
)
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import (
    UserJoinedVoiceChannelRepositoryABC,
)
from bot_data.abc.user_repository_abc import UserRepositoryABC
from bot_data.abc.user_warnings_repository_abc import UserWarningsRepositoryABC
from bot_data.model.short_role_name_position_enum import ShortRoleNamePositionEnum
from bot_data.model.technician_config import TechnicianConfig
from bot_graphql.abc.query_abc import QueryABC
from bot_graphql.filter.achievement_filter import AchievementFilter
from bot_graphql.filter.auto_role_filter import AutoRoleFilter
from bot_graphql.filter.auto_role_rule_filter import AutoRoleRuleFilter
from bot_graphql.filter.client_filter import ClientFilter
from bot_graphql.filter.level_filter import LevelFilter
from bot_graphql.filter.scheduled_event_filter import ScheduledEventFilter
from bot_graphql.filter.server_filter import ServerFilter
from bot_graphql.filter.short_role_name_filter import ShortRoleNameFilter
from bot_graphql.filter.user_filter import UserFilter
from bot_graphql.filter.user_joined_game_server_filter import UserJoinedGameServerFilter
from bot_graphql.filter.user_joined_server_filter import UserJoinedServerFilter
from bot_graphql.filter.user_joined_voice_channel_filter import (
    UserJoinedVoiceChannelFilter,
)
from bot_graphql.filter.user_warning_filter import UserWarningFilter
from bot_graphql.model.discord import Discord
from modules.achievements.achievement_service import AchievementService


class Query(QueryABC):
    def __init__(
        self,
        config: ConfigurationABC,
        bot: DiscordBotServiceABC,
        auto_roles: AutoRoleRepositoryABC,
        clients: ClientRepositoryABC,
        known_users: KnownUserRepositoryABC,
        levels: LevelRepositoryABC,
        servers: ServerRepositoryABC,
        game_servers: GameServerRepositoryABC,
        user_joined_servers: UserJoinedServerRepositoryABC,
        user_joined_voice_channels: UserJoinedVoiceChannelRepositoryABC,
        user_joined_game_server: UserJoinedGameServerRepositoryABC,
        users: UserRepositoryABC,
        achievements: AchievementRepositoryABC,
        short_role_names: ShortRoleNameRepositoryABC,
        user_warnings: UserWarningsRepositoryABC,
        achievement_service: AchievementService,
        technician_config: TechnicianConfigRepositoryABC,
        scheduled_events: ScheduledEventRepositoryABC,
    ):
        QueryABC.__init__(self, "Query")

        self._config = config

        self.add_collection("autoRole", lambda *_: auto_roles.get_auto_roles(), AutoRoleFilter)
        self.add_collection(
            "autoRoleRule",
            lambda *_: auto_roles.get_auto_role_rules(),
            AutoRoleRuleFilter,
        )
        self.add_collection("client", lambda *_: clients.get_clients(), ClientFilter)
        self.add_collection("knownUser", lambda *_: known_users.get_users())
        self.add_collection("level", lambda *_: levels.get_levels(), LevelFilter)
        self.add_collection("server", lambda *_: servers.get_servers(), ServerFilter)
        self.add_collection("gameServer", lambda *_: game_servers.get_game_servers())
        self.add_collection(
            "userJoinedServer",
            lambda *_: user_joined_servers.get_user_joined_servers(),
            UserJoinedServerFilter,
        )
        self.add_collection(
            "userJoinedVoiceChannel",
            lambda *_: user_joined_voice_channels.get_user_joined_voice_channels(),
            UserJoinedVoiceChannelFilter,
        )
        self.add_collection(
            "userJoinedGameServer",
            lambda *_: user_joined_game_server.get_user_joined_game_servers(),
            UserJoinedGameServerFilter,
        )
        self.add_collection("user", lambda *_: users.get_users(), UserFilter)
        self.add_collection("achievement", lambda *_: achievements.get_achievements(), AchievementFilter)
        self.add_collection(
            "shortRoleName",
            lambda *_: short_role_names.get_short_role_names(),
            ShortRoleNameFilter,
        )
        self.add_collection(
            "scheduledEvent",
            lambda server, *_: scheduled_events.get_scheduled_events_by_server_id(server.id),
            ScheduledEventFilter,
        )
        self.add_collection(
            "userWarning",
            lambda *_: user_warnings.get_user_warnings(),
            UserWarningFilter,
        )

        self.set_field("technicianConfig", lambda *_: technician_config.get_technician_config())

        self.set_field("achievementAttributes", lambda *_: achievement_service.get_attributes())
        self.set_field("achievementOperators", lambda *_: achievement_service.get_operators())
        self.set_field(
            "shortRoleNamePositions",
            lambda *_: [x.value for x in ShortRoleNamePositionEnum],
        )

        self.set_field("possibleFeatureFlags", lambda *_: [e.value for e in FeatureFlagsEnum])
        self.set_field("discord", lambda *_: Discord(bot.guilds, List(any).extend(bot.users)))

        self.set_field(
            "hasFeatureFlag",
            lambda *_, **kwargs: self._resolve_has_feature_flag(*_, **kwargs),
        )
        self.set_field(
            "featureFlags",
            lambda *_, **kwargs: self._resolve_feature_flags(*_),
        )

    def _resolve_has_feature_flag(self, *_, **kwargs):
        settings: TechnicianConfig = self._config.get_configuration(TechnicianConfig)
        if "flag" not in kwargs:
            return False
        return {
            "key": kwargs["flag"],
            "value": FeatureFlagsSettings.get_flag_from_dict(settings.feature_flags, FeatureFlagsEnum(kwargs["flag"])),
        }

    def _resolve_feature_flags(self, *_) -> list[dict]:
        settings: TechnicianConfig = self._config.get_configuration(TechnicianConfig)
        return [
            {
                "key": flag,
                "value": FeatureFlagsSettings.get_flag_from_dict(settings.feature_flags, FeatureFlagsEnum(flag)),
            }
            for flag in [e.value for e in FeatureFlagsEnum]
        ]
