import { Component, EventEmitter, Input, OnChanges, OnInit, Output } from "@angular/core";
import {
  EventType,
  ScheduledEvent,
  ScheduledEventInterval
} from "../../../../../../models/data/scheduled_events.model";
import { TranslateService } from "@ngx-translate/core";
import { FormBuilder, FormControl, FormGroup, Validators } from "@angular/forms";
import { SingleDiscordQuery } from "../../../../../../models/graphql/query.model";
import { Queries } from "../../../../../../models/graphql/queries.model";
import { Channel, ChannelType, Guild } from "../../../../../../models/data/discord.model";
import { DataService } from "../../../../../../services/data/data.service";
import { SpinnerService } from "../../../../../../services/spinner/spinner.service";
import { ActivatedRoute } from "@angular/router";
import { Server } from "../../../../../../models/data/server.model";
import { DatePipe } from "@angular/common";

@Component({
  selector: "app-edit-scheduled-event-dialog",
  templateUrl: "./edit-scheduled-event-dialog.component.html",
  styleUrls: ["./edit-scheduled-event-dialog.component.scss"]
})
export class EditScheduledEventDialogComponent implements OnInit, OnChanges {
  public _event?: ScheduledEvent;

  @Input()
  set event(event: ScheduledEvent | undefined) {
    this._event = event;
    this.eventChange.emit(event);
  }

  get event(): ScheduledEvent | undefined {
    return this._event;
  }

  @Output() eventChange: EventEmitter<ScheduledEvent> = new EventEmitter<ScheduledEvent>();

  get header() {
    if (this.event && this.event.createdAt === "" && this.event.modifiedAt === "") {
      return this.translate.instant("view.server.scheduled_events.edit_dialog.add_header");
    } else {
      return this.translate.instant("view.server.scheduled_events.edit_dialog.edit_header");
    }
  }

  get visible() {
    return this.event != undefined;
  }

  set visible(val: boolean) {
    if (!val) {
      this.event = undefined;
      this.inputForm.reset();
      this.activeIndex = 0;
    }
  }

  public activeIndex: number = 0;
  public inputForm!: FormGroup<{
    entityType: FormControl<number | undefined | null>;
    name: FormControl<string | undefined | null>;
    description: FormControl<string | undefined | null>;
    interval: FormControl<ScheduledEventInterval | undefined | null>;
    location: FormControl<string | undefined | null>;
    startTime: FormControl<Date | undefined | null>;
    id: FormControl<number | undefined | null>;
    endTime: FormControl<Date | undefined | null>;
    channelId: FormControl<string | undefined | null>
  }>;
  server: Server = {};
  public voiceChannels: Channel[] = [];
  public stageChannels: Channel[] = [];
  public guild: Guild = { channels: [], emojis: [], roles: [] };
  public times: string[] = [];
  public now = new Date();
  public interval = [
    {
      label: this.translate.instant("view.server.scheduled_events.edit_dialog.location.intervals.daily"),
      value: "daily"
    },
    {
      label: this.translate.instant("view.server.scheduled_events.edit_dialog.location.intervals.weekly"),
      value: "weekly"
    },
    {
      label: this.translate.instant("view.server.scheduled_events.edit_dialog.location.intervals.monthly"),
      value: "monthly"
    },
    {
      label: this.translate.instant("view.server.scheduled_events.edit_dialog.location.intervals.yearly"),
      value: "yearly"
    }
  ];

  constructor(
    private translate: TranslateService,
    private fb: FormBuilder,
    private data: DataService,
    private spinner: SpinnerService,
    private route: ActivatedRoute,
    private datePipe: DatePipe
  ) {
    for (let i = 0; i < 25; i++) {
      let time = "";
      if (i < 10) {
        time = `0${i}`;
      } else {
        time = `${i}`;
      }
      this.times.push(`${time}:00`);
      this.times.push(`${time}:15`);
      this.times.push(`${time}:30`);
      this.times.push(`${time}:45`);
    }
    this.setInputForm();
  }

  public ngOnChanges() {
    this.setInputForm();
  }

  public ngOnInit() {
    this.data.getServerFromRoute(this.route).then(server => {
      this.server = server;
      this.spinner.showSpinner();

      this.data.query<SingleDiscordQuery>(Queries.guildsQuery, {
          id: server?.discordId
        }
      ).subscribe(data => {
        if (data.discord.guilds) {
          this.guild = data.discord.guilds[0];
        }

        this.voiceChannels = this.guild.channels.filter(x => x.type === ChannelType.voice);
        this.stageChannels = this.guild.channels.filter(x => x.type === ChannelType.stage);
        this.setInputForm();
        this.spinner.hideSpinner();
      });
    });
  }

  public setInputForm() {
    if (this.now.getMinutes() % 15 != 0) {
      if (this.now.getMinutes() < 15) {
        this.now.setMinutes(15);
      } else if (this.now.getMinutes() < 30) {
        this.now.setMinutes(30);
      } else if (this.now.getMinutes() < 45) {
        this.now.setMinutes(45);
      } else if (this.now.getMinutes() > 45) {
        this.now.setMinutes(0);
        this.now.setHours(this.now.getHours() + 1);
      }
    }
    this.now.setSeconds(0);

    this.inputForm = this.fb.group({
      id: new FormControl<number | undefined>(this.event?.id),
      interval: new FormControl<ScheduledEventInterval | undefined>(this.event?.interval, [Validators.required]),
      entityType: new FormControl<number | undefined>(this.event?.entityType, [Validators.required]),
      channelId: new FormControl<string | undefined>(this.event?.channelId, this.event?.entityType == EventType.voice || this.event?.entityType == EventType.stageInstance ? [Validators.required] : []),
      location: new FormControl<string | undefined>(this.event?.location, this.event?.entityType == EventType.external ? [Validators.required] : []),
      name: new FormControl<string | undefined>(this.event?.name, [Validators.required]),
      startTime: new FormControl<Date | undefined>(this.event?.startTime ? new Date(this.event.startTime) : this.now, [Validators.required]),
      endTime: new FormControl<Date | undefined>(this.event?.endTime ? new Date(this.event.endTime) : undefined),
      description: new FormControl<string | undefined>(this.event?.description)
    });
  }

  public saveEvent() {
    this.event = {
      id: this.inputForm.controls.id.value ?? undefined,
      interval: this.inputForm.controls.interval.value ?? undefined,
      name: this.inputForm.controls.name.value ?? undefined,
      description: this.inputForm.controls.description.value ?? undefined,
      channelId: this.inputForm.controls.channelId.value ?? undefined,
      startTime: this.inputForm.controls.startTime.value ?? undefined,
      endTime: this.inputForm.controls.endTime.value ?? undefined,
      entityType: this.inputForm.controls.entityType.value ?? undefined,
      location: this.inputForm.controls.location.value ?? undefined,
      server: this.server
    };
    this.visible = false;
  }

  public next() {
    this.activeIndex++;
  }

  public back() {
    this.activeIndex--;
  }

  protected readonly EventType = {
    stage: EventType.stageInstance,
    voice: EventType.voice,
    somewhere_else: EventType.external
  };
}
